# coding: utf-8

"""
    Machine Catalogue

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from inductiva.machines_catalogue_client.models.providers import Providers
from typing import Optional, Set
from typing_extensions import Self


class MachineTypeWithSpotPrice(BaseModel):
    """
    MachineTypeWithSpotPrice
    """

  # noqa: E501
    machine_type: StrictStr
    num_vcpus: StrictInt
    ram_gb: Union[StrictFloat, StrictInt]
    price: Union[StrictFloat, StrictInt]
    provider_id: Optional[Providers] = None
    threads_per_core: Optional[StrictInt] = None
    spot: Optional[StrictBool] = None
    region: Optional[StrictStr] = None
    zone: Optional[StrictStr] = None
    num_gpus: Optional[StrictInt] = None
    gpu_id: Optional[StrictStr] = None
    gpu_name: Optional[StrictStr] = None
    spot_price: Optional[Union[StrictFloat, StrictInt]] = None
    __properties: ClassVar[List[str]] = [
        "machine_type", "num_vcpus", "ram_gb", "price", "provider_id",
        "threads_per_core", "spot", "region", "zone", "num_gpus", "gpu_id",
        "gpu_name", "spot_price"
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of MachineTypeWithSpotPrice from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if threads_per_core (nullable) is None
        # and model_fields_set contains the field
        if self.threads_per_core is None and "threads_per_core" in self.model_fields_set:
            _dict['threads_per_core'] = None

        # set to None if spot (nullable) is None
        # and model_fields_set contains the field
        if self.spot is None and "spot" in self.model_fields_set:
            _dict['spot'] = None

        # set to None if region (nullable) is None
        # and model_fields_set contains the field
        if self.region is None and "region" in self.model_fields_set:
            _dict['region'] = None

        # set to None if zone (nullable) is None
        # and model_fields_set contains the field
        if self.zone is None and "zone" in self.model_fields_set:
            _dict['zone'] = None

        # set to None if num_gpus (nullable) is None
        # and model_fields_set contains the field
        if self.num_gpus is None and "num_gpus" in self.model_fields_set:
            _dict['num_gpus'] = None

        # set to None if gpu_id (nullable) is None
        # and model_fields_set contains the field
        if self.gpu_id is None and "gpu_id" in self.model_fields_set:
            _dict['gpu_id'] = None

        # set to None if gpu_name (nullable) is None
        # and model_fields_set contains the field
        if self.gpu_name is None and "gpu_name" in self.model_fields_set:
            _dict['gpu_name'] = None

        # set to None if spot_price (nullable) is None
        # and model_fields_set contains the field
        if self.spot_price is None and "spot_price" in self.model_fields_set:
            _dict['spot_price'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of MachineTypeWithSpotPrice from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "machine_type": obj.get("machine_type"),
            "num_vcpus": obj.get("num_vcpus"),
            "ram_gb": obj.get("ram_gb"),
            "price": obj.get("price"),
            "provider_id": obj.get("provider_id"),
            "threads_per_core": obj.get("threads_per_core"),
            "spot": obj.get("spot"),
            "region": obj.get("region"),
            "zone": obj.get("zone"),
            "num_gpus": obj.get("num_gpus"),
            "gpu_id": obj.get("gpu_id"),
            "gpu_name": obj.get("gpu_name"),
            "spot_price": obj.get("spot_price")
        })
        return _obj



"""Remove the user's remote storage contents via CLI."""
import argparse
import textwrap

import inductiva
from ...localization import translator as __
from inductiva.client import exceptions
from inductiva.utils.input_functions import user_confirmation_prompt


def remove(args):
    """Remove user's remote storage contents."""
    if not args.confirm:
        confirm = user_confirmation_prompt(
            args.paths,
            __("storage-prompt-remove-all"),
            __("storage-prompt-remove-big", len(args.paths)),
            __("storage-prompt-remove-small"),
            args.all,
        )
        if not confirm:
            return

    deletion_list = []

    if args.all:
        root_dir = inductiva.storage.listdir(max_results=None, region="all")
        deletion_list = [
            (directory["name"], directory["region"]) for directory in root_dir
        ]
    else:
        for path in args.paths:
            deletion_list.append((path, args.region))

    for path, region in deletion_list:
        try:
            inductiva.storage.remove(remote_path=path, region=region)
        except exceptions.ApiException as e:
            print("\nError:", str(e))


def register(parser):
    subparser = parser.add_parser(
        "remove",
        aliases=["rm"],
        help="Remove files or directories from remote storage.",
        formatter_class=argparse.RawTextHelpFormatter)
    subparser.description = (
        "The `inductiva storage remove` command deletes specific files or"
        "folders from your Inductiva remote storage.\n"
        "Use it with caution. This action is irreversible and will permanently "
        "remove the selected files or directories from your remote storage.\n")

    subparser.add_argument(
        "paths",
        type=str,
        nargs="*",
        help="Remote path(s) to remove from storage.",
    )

    subparser.add_argument(
        "-y",
        "--yes",
        action="store_true",
        dest="confirm",
        default=False,
        help=("Sets any confirmation values to \"yes\" automatically. Users "
              "will not be asked for confirmation to remove path(s) from "
              "remote storage."),
    )

    subparser.add_argument(
        "-a",
        "--all",
        action="store_true",
        default=False,
        help="Remove all data from all remote storage regions.",
    )

    subparser.add_argument(
        "-r",
        "--region",
        default=None,
        type=str,
        help=("Storage region of Inductiva remote files. If not specified, "
              "the user's default region is assumed. If --all flag is "
              "also specified this value will be ignored."),
    )

    subparser.epilog = textwrap.dedent("""\
        examples:
            $ inductiva storage remove 0bet8jrpp2gz974n42nsd9n2p/
            You are about to remove the following paths from your remote storage space:
            - 0bet8jrpp2gz974n42nsd9n2p/
            Are you sure you want to proceed (y/[N])? y
            Removing '0bet8jrpp2gz974n42nsd9n2p/' from remote storage...
            Successfully removed '0bet8jrpp2gz974n42nsd9n2p/' from remote storage.
    """)

    subparser.set_defaults(func=remove)



"""CalculiX module of the API for numerical simulations of fluid flows."""
from typing import Optional, Union

from inductiva import simulators, tasks, types
from inductiva.commands import Command


class CalculiX(simulators.Simulator):
    """Class to invoke a generic CalculiX simulation on the API.
    """

    def __init__(self, /, version: Optional[str] = None, use_dev: bool = False):
        """Initialize the CalculiX simulator.

        Args:
            version (str): The version of the simulator to use. If None, the
                latest available version in the platform is used.
            use_dev (bool): Request use of the development version of
                the simulator. By default (False), the production version
                is used.
        """
        super().__init__(version=version, use_dev=use_dev)
        self.simulator = "arbitrary_commands"
        self.simulator_name_alias = "calculix"

    def run(self,
            input_dir: Optional[str],
            *,
            sim_config_filename: str,
            on: types.ComputationalResources,
            n_vcpus: Optional[int] = None,
            storage_dir: Optional[str] = "",
            resubmit_on_preemption: bool = False,
            remote_assets: Optional[Union[str, list[str]]] = None,
            project: Optional[str] = None,
            time_to_live: Optional[str] = None,
            on_finish_cleanup: Optional[Union[str, list[str]]] = None,
            **kwargs) -> tasks.Task:
        """Run the simulation.

        Args:
            input_dir: Path to the directory of the simulation input files.
            sim_config_filename: Name of the simulation configuration file.
            on: The computational resource to launch the simulation on.
            n_vcpus: Number of vCPUs to use in the simulation. If not provided
                (default), all vCPUs will be used.
            storage_dir: Path to the directory where the simulation results
                will be stored.
            resubmit_on_preemption: Resubmit task for execution when
                previous execution attempts were preempted. Only applicable when
                using a preemptible resource, i.e., resource instantiated with
                `spot=True`.
            remote_assets: Additional remote files that will be copied to
                the simulation directory.
            project: Name of the project to which the task will be
                assigned. If None, the task will be assigned to
                the default project. If the project does not exist, it will be
                created.
            time_to_live: Maximum allowed runtime for the task, specified as a
                string duration. Supports common time duration formats such as
                "10m", "2 hours", "1h30m", or "90s". The task will be
                automatically terminated if it exceeds this duration after
                starting.
            on_finish_cleanup :
                Optional cleanup script or list of shell commands to remove
                temporary or unwanted files generated during the simulation.
                This helps reduce storage usage by discarding unnecessary
                output.
                - If a string is provided, it is treated as the path to a shell
                script that must be included with the simulation files.
                - If a list of strings is provided, each item is treated as an
                individual shell command and will be executed sequentially.
                All cleanup actions are executed in the simulation's working
                directory, after the simulation finishes.
                Examples:
                    on_finish_cleanup = "my_cleanup.sh"

                    on_finish_cleanup = [
                        "rm -rf temp_dir",
                        "rm -f logs/debug.log"
                    ]
            **kwargs: Keyword arguments to be passed to the base class.
        """

        self._input_files_exist(input_dir=input_dir,
                                remote_assets=remote_assets,
                                sim_config_filename=sim_config_filename)

        if n_vcpus is None:
            n_vcpus = on.available_vcpus

        commands = [
            Command(f"ccx -i {sim_config_filename[:-4]}",
                    env={
                        "OMP_NUM_THREADS": str(n_vcpus),
                    })
        ]

        return super().run(input_dir,
                           on=on,
                           storage_dir=storage_dir,
                           commands=commands,
                           resubmit_on_preemption=resubmit_on_preemption,
                           remote_assets=remote_assets,
                           project=project,
                           time_to_live=time_to_live,
                           on_finish_cleanup=on_finish_cleanup,
                           **kwargs)



"""Register CLI commands for logs."""
import argparse
import os
import textwrap

from .. import loader, utils
from ... import constants


def register(root_parser):
    parser = root_parser.add_parser(
        "auth",
        help="Authentication commands",
        formatter_class=argparse.RawTextHelpFormatter)

    parser.description = textwrap.dedent("""\
        Authentication management utilities.

        The `inductiva auth` command allows you to manage your authentication
        on Inductiva API.

        This is a fundamental aspect of your experience: you will only be able 
        to start machines and launch simulations after you are authenticated. 
        Once authenticated, your credentials will be stored locally for future 
        sessions.
                                         
        However, you will need to authenticate separately on each local machine
        from which you want to use Inductiva.
    """)

    utils.show_help_msg(parser)

    subparsers = parser.add_subparsers(title="available subcomands")
    loader.load_commands(subparsers,
                         os.path.dirname(__file__),
                         package=__name__,
                         ignores_prefix=constants.LOADER_IGNORE_PREFIX)



# coding: utf-8

# flake8: noqa

"""
    Machine Catalogue

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

__version__ = "1.0.0"

# import apis into sdk package
from inductiva.machines_catalogue_client.api.compute_api import ComputeApi
from inductiva.machines_catalogue_client.api.metrics_api import MetricsApi
from inductiva.machines_catalogue_client.api.pricing_api import PricingApi
from inductiva.machines_catalogue_client.api.simulators_api import SimulatorsApi

# import ApiClient
from inductiva.machines_catalogue_client.api_response import ApiResponse
from inductiva.machines_catalogue_client.api_client import ApiClient
from inductiva.machines_catalogue_client.configuration import Configuration
from inductiva.machines_catalogue_client.exceptions import OpenApiException
from inductiva.machines_catalogue_client.exceptions import ApiTypeError
from inductiva.machines_catalogue_client.exceptions import ApiValueError
from inductiva.machines_catalogue_client.exceptions import ApiKeyError
from inductiva.machines_catalogue_client.exceptions import ApiAttributeError
from inductiva.machines_catalogue_client.exceptions import ApiException

# import models into sdk package
from inductiva.machines_catalogue_client.models.available_machine_series import AvailableMachineSeries
from inductiva.machines_catalogue_client.models.available_simulator_versions import AvailableSimulatorVersions
from inductiva.machines_catalogue_client.models.available_zones import AvailableZones
from inductiva.machines_catalogue_client.models.cloud_location import CloudLocation
from inductiva.machines_catalogue_client.models.http_validation_error import HTTPValidationError
from inductiva.machines_catalogue_client.models.machine_filter_options import MachineFilterOptions
from inductiva.machines_catalogue_client.models.machine_type import MachineType
from inductiva.machines_catalogue_client.models.machine_type_with_spot_price import MachineTypeWithSpotPrice
from inductiva.machines_catalogue_client.models.preemption_stats import PreemptionStats
from inductiva.machines_catalogue_client.models.processor_type import ProcessorType
from inductiva.machines_catalogue_client.models.product_price import ProductPrice
from inductiva.machines_catalogue_client.models.providers import Providers
from inductiva.machines_catalogue_client.models.simulator_prod_update import SimulatorProdUpdate
from inductiva.machines_catalogue_client.models.simulator_response import SimulatorResponse
from inductiva.machines_catalogue_client.models.simulator_version import SimulatorVersion
from inductiva.machines_catalogue_client.models.simulator_with_machine_types_paginated import SimulatorWithMachineTypesPaginated
from inductiva.machines_catalogue_client.models.spot import Spot
from inductiva.machines_catalogue_client.models.validation_error import ValidationError
from inductiva.machines_catalogue_client.models.validation_error_loc_inner import ValidationErrorLocInner
from inductiva.machines_catalogue_client.models.validation_result import ValidationResult
from inductiva.machines_catalogue_client.models.weekly_preemption_stats import WeeklyPreemptionStats



"""Download storage contents via CLI."""

import argparse
import textwrap
from inductiva import storage


def download(args):
    """Download a file or folder from remote storage."""
    storage.download(
        remote_path=args.remote_path,
        local_dir=args.local_dir,
        decompress=args.decompress,
        region=args.region,
    )


def register(parser):
    """Register the download command for the user's remote storage."""

    subparser = parser.add_parser(
        "download",
        aliases=["dl"],
        help="Download a file or folder from remote storage.",
        formatter_class=argparse.RawTextHelpFormatter,
    )

    subparser.description = (
        "The `inductiva storage download` command allows you to download a file"
        " or folder from your remote storage, maintaining the original storage "
        "path structure, to your local machine.\n"
        "Specify the remote path, the local destination (optional), and whether"
        " to decompress the content after downloading (default: enabled).\n")

    subparser.add_argument(
        "remote_path",
        type=str,
        help="The path to the file or folder in remote storage to download.",
    )

    subparser.add_argument(
        "local_dir",
        type=str,
        nargs="?",
        default="",
        help=("The local directory where the downloaded content will be saved. "
              "Defaults to the current working directory if not specified."),
    )

    subparser.add_argument(
        "--decompress",
        action="store_true",
        help="Decompress the downloaded file or folder if it is compressed.",
    )

    subparser.add_argument(
        "-r",
        "--region",
        default=None,
        type=str,
        help=("Storage region of remote files. If not specified, the user's "
              "default region is assumed."),
    )

    subparser.epilog = textwrap.dedent("""\
        examples:
            # Download and decompress an archived file
            $ inductiva storage download my_data/archive.zip --decompress
    """)

    subparser.set_defaults(func=download)



"""Functions to manage or retrieve user resources."""
from dataclasses import dataclass
from typing import List, Optional, Tuple, Union

import inductiva
import inductiva.client
import inductiva.machines_catalogue_client
from inductiva.utils import format_utils


class ProviderType(format_utils.CaseInsensitiveEnum):
    """Enum to represent the provider of the machine to be launched."""
    GCP = "GCP"
    LOCAL = "LOCAL"


@dataclass(frozen=True)
class MachineTypeInfo:
    """Dataclass to represent a machine type.
    This class has all the information related to a machine type.
    """

    threads_per_core: int
    machine_type: str
    provider_id: str
    num_vcpus: int
    price: float
    ram_gb: int
    region: str
    spot: bool
    zone: str
    num_gpus: Optional[int]
    gpu_name: Optional[str]
    gpu_id: Optional[str]

    def __getitem__(self, item):
        return getattr(self, item)


def get_available_machine_types(
    provider: Union[str, ProviderType] = ProviderType.GCP,
    machine_families: Optional[List[str]] = None,
    machine_configs: Optional[List[str]] = None,
    vcpus_range: Optional[Tuple[int, int]] = None,
    memory_range: Optional[Tuple[int, int]] = None,
    price_range: Optional[Tuple[float, float]] = None,
    spot: Optional[bool] = None,
    gpus_range: Optional[Tuple[int, int]] = None,
    gpu_names: Optional[List[str]] = None,
    zones: Optional[List[str]] = None,
) -> List[MachineTypeInfo]:
    """Get all available machine types from a specified provider,
    allowing for multiple filtering parameters.

    Args:
        provider (Union[str, ProviderType]):
            The cloud provider for which to list available machine types.
            Defaults to `ProviderType.GCP`.
        machine_families (Optional[List[str]]):
            A list of machine families to filter the available machine types
            (e.g., "c2", "c2d", "n4", etc).
        machine_configs (Optional[List[str]]):
            A list of specific machine configurations to filter the results
            (e.g., "highcpu", "highmem", "standard").
        vcpus_range (Optional[Tuple[int, int]]):
            A tuple defining the range of virtual CPUs (vCPUs) to filter the
            machine types.
        memory_range (Optional[Tuple[int, int]]):
            A tuple defining the range of memory (in MB) to filter the machine
            types.
        price_range (Optional[Tuple[float, float]]):
            A tuple defining the price range (in the respective currency) to
            filter the machine types.
        spot (Optional[bool]):
            If set to True, filters for spot instances; if False, filters for
            on-demand instances.
        gpus_range (Optional[Tuple[int, int]]):
            A tuple defining the range of GPUs to filter the machine types.
        gpu_names (Optional[List[str]]):
            A list of GPU model names to filter the machine types
            (e.g., "NVIDIA H100", "NVIDIA_A100").
        zones (Optional[List[str]]):
            A list of availability zones to filter the machine types (e.g., 
            "us-central1-a", "europe-west1-b").

    Returns:
        List[MachineTypeInfo]:
            A list of available machine types that match the specified criteria.
    """
    client = inductiva.api.get_machines_catalogue_client()
    api = inductiva.machines_catalogue_client.PricingApi(client)

    provider = ProviderType(provider)
    if spot is not None:
        spot = str(spot).lower()

    machine_types = api.list_available_machine_types_pricing_machine_types_get(
        provider_id=provider,
        machine_families=machine_families,
        machine_configs=machine_configs,
        vcpus_range=vcpus_range,
        memory_range=memory_range,
        price_range=price_range,
        spot=spot,
        gpus_range=gpus_range,
        gpu_names=gpu_names,
        zones=zones,
    )
    return [MachineTypeInfo(**mt.to_dict()) for mt in machine_types]


def estimate_machine_cost(machine_type: str,
                          spot: bool = False,
                          zone: str = None):
    """Estimate the cloud cost of one machine per hour in US dollars.

    Args:
        machine_type: The type of GC machine to launch. Ex: "e2-standard-4".
            Check https://cloud.google.com/compute/docs/machine-resource for
            more information about machine types.
        zone: The zone where the machines will be launched.
        spot: Whether to use spot machines.
    """

    api = inductiva.client.ComputeApi(inductiva.api.get_client())

    instance_price = api.get_instance_price(machine_type=machine_type,
                                            zone=zone)

    if spot:
        estimated_cost = instance_price.get("preemptible_price")
    else:
        estimated_cost = instance_price.get("on_demand_price")

    return float(estimated_cost)



"""Methods to interact with the available simulators."""
from typing import Dict, List
import inductiva
import inductiva.machines_catalogue_client


def list_available_images() -> Dict[str, Dict[str, List[str]]]:
    """Fetch the list of available simulator images from the API."""
    client = inductiva.api.get_machines_catalogue_client()
    api = inductiva.machines_catalogue_client.SimulatorsApi(client)
    response = api.list_available_images_simulators_available_images_get()
    return response.model_dump()



"""Register CLI commands for containers."""
import argparse
import os
import textwrap

from .. import loader, utils
from ... import constants


def register(root_parser):
    parser = root_parser.add_parser(
        "containers",
        help="Containers commands",
        formatter_class=argparse.RawTextHelpFormatter)

    parser.description = textwrap.dedent("""\
        Manage custom simulation containers.

        The `inductiva containers` command provides utilities for managing
        user-defined containers, including converting Docker images to
        Apptainer-compatible  `.sif` files and uploading them to your private
        Inductiva remote storage for use in simulations.
    """)
    utils.show_help_msg(parser)

    subparsers = parser.add_subparsers(title="available subcomands")
    loader.load_commands(subparsers,
                         os.path.dirname(__file__),
                         package=__name__,
                         ignores_prefix=constants.LOADER_IGNORE_PREFIX)



"""List the available simulators via CLI."""
import argparse
import textwrap

from inductiva.simulators import methods
from inductiva.utils import format_utils


def list_versions(args):
    """List available simulators and associated versions for each branch."""

    available = methods.list_available_images()

    if not args.dev:
        available.pop("development")
    fmtr = format_utils.get_ansi_formatter()

    def header_fmtr(s):
        return fmtr(s.upper(), format_utils.Emphasis.BOLD)

    for branch, simulators in available.items():
        sims = sorted(simulators.keys())

        data = {
            "simulator": sims,
            "versions": [", ".join(simulators[s]) for s in sims],
        }

        table = format_utils.get_tabular_str(data,
                                             header_formatters=[header_fmtr])
        header = f"Available simulators and versions for {branch} runs:"

        print(header_fmtr(header))
        print(table)


def register(parser):
    """Register the "simulators list" sub-command."""

    subparser = parser.add_parser("list",
                                  aliases=["ls"],
                                  help="List available simulators.",
                                  formatter_class=argparse.RawTextHelpFormatter)

    subparser.description = textwrap.dedent("""\
        The `inductiva simulators list` sub-command lists all available
        simulators and associated versions for use with the Inductiva API,
        including those available for development purposes.
    """)

    subparser.add_argument(
        "--dev",
        action="store_true",
        help="include development versions of the simulators.")

    subparser.epilog = textwrap.dedent("""\
        examples:
            $ inductiva simulators list
            AVAILABLE SIMULATORS AND VERSIONS FOR PRODUCTION RUNS:

            SIMULATOR             VERSIONS
            amr-wind              1.4.0, 1.4.0_gpu, 3.4.0, 3.4.0_gpu, 3.4.1, 3.4.1_gpu
            apptainer-converter   0.1.0
            cans                  2.3.4, 2.4.0, 2.4.0_gpu, 3.0.0, 3.0.0_gpu
            cm1                   18, 21.1
            coawst                3.8
            cp2k                  2025.1, 2025.1_gpu
            delft3d               6.04.00
            dualsphysics          5.2.1, 5.2.1_gpu, 5.4.1, 5.4.1_gpu
            ...
    """)

    subparser.set_defaults(func=list_versions)



# coding: utf-8

"""
    Machine Catalogue

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from datetime import datetime
from pydantic import StrictStr
from typing import Any, List, Optional
from inductiva.machines_catalogue_client.models.preemption_stats import PreemptionStats
from inductiva.machines_catalogue_client.models.weekly_preemption_stats import WeeklyPreemptionStats

from inductiva.machines_catalogue_client.api_client import ApiClient, RequestSerialized
from inductiva.machines_catalogue_client.api_response import ApiResponse
from inductiva.machines_catalogue_client.rest import RESTResponseType


class MetricsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    def get_weekly_preemption_stats_metrics_preemption_stats_weekly_get(
        self,
        start_date: datetime,
        end_date: datetime,
        machine_series: Optional[List[StrictStr]] = None,
        zones: Optional[List[StrictStr]] = None,
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[WeeklyPreemptionStats]:
        """Get Weekly Preemption Stats

        Retrieve hourly preemption records for the current week.

        :param start_date: (required)
        :type start_date: datetime
        :param end_date: (required)
        :type end_date: datetime
        :param machine_series:
        :type machine_series: List[str]
        :param zones:
        :type zones: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_weekly_preemption_stats_metrics_preemption_stats_weekly_get_serialize(
            start_date=start_date,
            end_date=end_date,
            machine_series=machine_series,
            zones=zones,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[WeeklyPreemptionStats]",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def get_weekly_preemption_stats_metrics_preemption_stats_weekly_get_with_http_info(
        self,
        start_date: datetime,
        end_date: datetime,
        machine_series: Optional[List[StrictStr]] = None,
        zones: Optional[List[StrictStr]] = None,
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[WeeklyPreemptionStats]]:
        """Get Weekly Preemption Stats

        Retrieve hourly preemption records for the current week.

        :param start_date: (required)
        :type start_date: datetime
        :param end_date: (required)
        :type end_date: datetime
        :param machine_series:
        :type machine_series: List[str]
        :param zones:
        :type zones: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_weekly_preemption_stats_metrics_preemption_stats_weekly_get_serialize(
            start_date=start_date,
            end_date=end_date,
            machine_series=machine_series,
            zones=zones,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[WeeklyPreemptionStats]",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def get_weekly_preemption_stats_metrics_preemption_stats_weekly_get_without_preload_content(
        self,
        start_date: datetime,
        end_date: datetime,
        machine_series: Optional[List[StrictStr]] = None,
        zones: Optional[List[StrictStr]] = None,
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Weekly Preemption Stats

        Retrieve hourly preemption records for the current week.

        :param start_date: (required)
        :type start_date: datetime
        :param end_date: (required)
        :type end_date: datetime
        :param machine_series:
        :type machine_series: List[str]
        :param zones:
        :type zones: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_weekly_preemption_stats_metrics_preemption_stats_weekly_get_serialize(
            start_date=start_date,
            end_date=end_date,
            machine_series=machine_series,
            zones=zones,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[WeeklyPreemptionStats]",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        return response_data.response

    def _get_weekly_preemption_stats_metrics_preemption_stats_weekly_get_serialize(
        self,
        start_date,
        end_date,
        machine_series,
        zones,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'machine_series': 'multi',
            'zones': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes, List[str], List[bytes],
                                List[Tuple[str, bytes]]]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if start_date is not None:
            if isinstance(start_date, datetime):
                _query_params.append(
                    ('start_date',
                     start_date.strftime(
                         self.api_client.configuration.datetime_format)))
            else:
                _query_params.append(('start_date', start_date))

        if end_date is not None:
            if isinstance(end_date, datetime):
                _query_params.append(
                    ('end_date',
                     end_date.strftime(
                         self.api_client.configuration.datetime_format)))
            else:
                _query_params.append(('end_date', end_date))

        if machine_series is not None:

            _query_params.append(('machine_series', machine_series))

        if zones is not None:

            _query_params.append(('zones', zones))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                ['application/json'])

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/metrics/preemption-stats/weekly',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth)

    @validate_call
    def post_preemption_stats_metrics_preemption_stats_post(
        self,
        preemption_stats: List[PreemptionStats],
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> object:
        """Post Preemption Stats

        Insert new hourly preemption records into the database.

        :param preemption_stats: (required)
        :type preemption_stats: List[PreemptionStats]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_preemption_stats_metrics_preemption_stats_post_serialize(
            preemption_stats=preemption_stats,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "object",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def post_preemption_stats_metrics_preemption_stats_post_with_http_info(
        self,
        preemption_stats: List[PreemptionStats],
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[object]:
        """Post Preemption Stats

        Insert new hourly preemption records into the database.

        :param preemption_stats: (required)
        :type preemption_stats: List[PreemptionStats]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_preemption_stats_metrics_preemption_stats_post_serialize(
            preemption_stats=preemption_stats,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "object",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def post_preemption_stats_metrics_preemption_stats_post_without_preload_content(
        self,
        preemption_stats: List[PreemptionStats],
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Post Preemption Stats

        Insert new hourly preemption records into the database.

        :param preemption_stats: (required)
        :type preemption_stats: List[PreemptionStats]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_preemption_stats_metrics_preemption_stats_post_serialize(
            preemption_stats=preemption_stats,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "object",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        return response_data.response

    def _post_preemption_stats_metrics_preemption_stats_post_serialize(
        self,
        preemption_stats,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'PreemptionStats': '',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes, List[str], List[bytes],
                                List[Tuple[str, bytes]]]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if preemption_stats is not None:
            _body_params = preemption_stats

        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                ['application/json'])

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (self.api_client.select_header_content_type(
                ['application/json']))
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/metrics/preemption-stats',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth)



# flake8: noqa

# import apis into api package
from inductiva.machines_catalogue_client.api.compute_api import ComputeApi
from inductiva.machines_catalogue_client.api.metrics_api import MetricsApi
from inductiva.machines_catalogue_client.api.pricing_api import PricingApi
from inductiva.machines_catalogue_client.api.simulators_api import SimulatorsApi



"""WAVEWATCH III module of the API for numerical simulations."""
from typing import Optional, Union

from inductiva import simulators, tasks, types


@simulators.simulator.mpi_enabled
class WaveWatch3(simulators.Simulator):
    """Class to invoke a generic WAVEWATCH III simulation on the API.
    """

    def __init__(self, /, version: Optional[str] = None, use_dev: bool = False):
        """Initialize the WAVEWATCH simulator.

        Args:
            version (str): The version of the simulator to use. If None, the
                latest available version in the platform is used.
            use_dev (bool): Request use of the development version of
                the simulator. By default (False), the production version
                is used.
        """
        super().__init__(version=version, use_dev=use_dev)
        self.simulator = "arbitrary_commands"
        self.simulator_name_alias = "WAVEWATCH-III"

    def run(self,
            input_dir: Optional[str],
            *,
            commands: types.Commands = None,
            shell_script: Optional[str] = None,
            switch: Optional[str] = None,
            custom_switch: Optional[str] = None,
            on: types.ComputationalResources,
            storage_dir: Optional[str] = "",
            resubmit_on_preemption: bool = False,
            remote_assets: Optional[Union[str, list[str]]] = None,
            project: Optional[str] = None,
            time_to_live: Optional[str] = None,
            on_finish_cleanup: Optional[Union[str, list[str]]] = None,
            **kwargs) -> tasks.Task:
        """Run the simulation.

        Args:
            input_dir: Path to the directory of the simulation input files.
            on: The computational resource to launch the simulation on.
            commands: List of commands to run the simulation. Can be used
                instead of `shell_scriot`.
            shell_script: Path to a shell script that will be executed to run
                the simulation. Can be used instead of `commands`.
            switch: The WAVEWATCH III switch to use for compilation. Some
                examples are: `Ifremer1`, `Ifremer2_pdlib`, `UKMO_reg` etc.
            custom_switch: Custom switch to use for compilation. Must be sent
                with the `input_dir`.
            resubmit_on_preemption (bool): Resubmit task for execution when
                previous execution attempts were preempted. Only applicable when
                using a preemptible resource, i.e., resource instantiated with
                `spot=True`.
            remote_assets: Additional remote files that will be copied to
                the simulation directory.
            project: Name of the project to which the task will be
                assigned. If None, the task will be assigned to
                the default project. If the project does not exist, it will be
                created.
            time_to_live: Maximum allowed runtime for the task, specified as a
                string duration. Supports common time duration formats such as
                "10m", "2 hours", "1h30m", or "90s". The task will be
                automatically terminated if it exceeds this duration after
                starting.
            on_finish_cleanup :
                Optional cleanup script or list of shell commands to remove
                temporary or unwanted files generated during the simulation.
                This helps reduce storage usage by discarding unnecessary
                output.
                - If a string is provided, it is treated as the path to a shell
                script that must be included with the simulation files.
                - If a list of strings is provided, each item is treated as an
                individual shell command and will be executed sequentially.
                All cleanup actions are executed in the simulation's working
                directory, after the simulation finishes.
                Examples:
                    on_finish_cleanup = "my_cleanup.sh"

                    on_finish_cleanup = [
                        "rm -rf temp_dir",
                        "rm -f logs/debug.log"
                    ]
        """
        compile_commands = []

        # Check if both commands and shell_script are provided or if neither is
        if (commands is None) == (shell_script is None):
            raise ValueError("You must provide exactly one of 'commands' or "
                             "'shell_script'.")

        # Check if switch and custom_switch are both provided or if neither is
        no_switches = switch is None and custom_switch is None
        both_switches = switch is not None and custom_switch is not None
        if (no_switches) or (both_switches):
            raise ValueError(
                "You must provide either 'switch' or 'custom_switch'.\n"
                "`switch` (Ex: `Ifremer1`, `Ifremer2_pdlib`, `UKMO_reg`) "
                "is a WAVEWATCH III switch and `custom_switch` is "
                "a custom switch provided with the input directory.")

        # Create the compilation commands based on the provided switch
        if switch is not None:
            compile_commands.append(f"bash /home/compile_ww3.sh {switch} false")
        if custom_switch is not None:
            compile_commands.append(
                f"bash /home/compile_ww3.sh {custom_switch} true")

        # Checks if the input files exist
        input_files_kwargs = {
            "input_dir": input_dir,
            "remote_assets": remote_assets,
        }
        if shell_script:
            input_files_kwargs["shell_script"] = shell_script
        if custom_switch:
            input_files_kwargs["custom_switch"] = custom_switch

        self._input_files_exist(**input_files_kwargs)

        commands = commands if commands is not None else [
            f"bash {shell_script}"
        ]
        commands = compile_commands + commands

        return super().run(input_dir,
                           on=on,
                           storage_dir=storage_dir,
                           commands=commands,
                           resubmit_on_preemption=resubmit_on_preemption,
                           remote_assets=remote_assets,
                           project=project,
                           time_to_live=time_to_live,
                           on_finish_cleanup=on_finish_cleanup,
                           **kwargs)



"""Register CLI commands for storage."""
import argparse
import os
import textwrap

from inductiva import constants
from inductiva._cli import loader, utils


def register(root_parser):

    parser = root_parser.add_parser(
        "tasks",
        help="Task management utilities.",
        formatter_class=argparse.RawTextHelpFormatter)

    parser.description = textwrap.dedent("""\
        Task management utilities.

        The `inductiva tasks` command provides tools to manage your tasks on 
        the Inductiva platform. It includes utilities for monitoring task 
        progress, viewing details, and terminating tasks when needed.
    """)

    utils.show_help_msg(parser)

    subparsers = parser.add_subparsers(title="available subcomands")
    loader.load_commands(subparsers,
                         os.path.dirname(__file__),
                         package=__name__,
                         ignores_prefix=constants.LOADER_IGNORE_PREFIX,
                         hides_prefix=constants.LOADER_HIDE_PREFIX)



"""CLI commands to terminate computational resources."""
import logging
import sys

import argparse
import textwrap

from inductiva import resources
import inductiva
from inductiva.utils import input_functions
from ...localization import translator as __


def terminate_machine_group(args):
    """Terminate one or all computational resources."""
    names = args.name
    all_names = args.all
    confirm = args.confirm

    if names and all_names:
        print(
            "inductiva resources terminate: error: "
            "argument name not allowed with argument --all",
            file=sys.stderr)
        return 1

    active_machines = resources.get()

    if not active_machines:
        print("No active resources to terminate.")
        return 0

    if not all_names and not names:
        print("No resource(s) specified.\n"
              "> Use `inductiva resources terminate -h` for help.")
        return 1

    # dict to map from name to machine
    name_to_machine = {machine.name: machine for machine in active_machines}
    # the user can give the same name multiple times
    target_machine_names = set(names or name_to_machine.keys())
    invalid_names = target_machine_names.difference(name_to_machine)

    if invalid_names:
        for name in invalid_names:
            print(f"Resource {name} does not exist.")
        print("Aborting.")
        return 1

    confirm = confirm or input_functions.user_confirmation_prompt(
        target_machine_names, __("resources-prompt-terminate-all"),
        __("resources-prompt-terminate-big", len(target_machine_names)),
        __("resources-prompt-terminate-small"), False)

    if not confirm:
        return 0

    print_quotas = False

    before_quotas = inductiva.users.get_quotas()
    for name in target_machine_names:
        name_to_machine[name].terminate(verbose=False)
        logging.info("Successfully requested termination of %s.",
                     name_to_machine[name].name)
        if (name_to_machine[name].provider ==
                inductiva.resources.utils.ProviderType.GCP):
            print_quotas = True

    if print_quotas:
        cls = inductiva.resources.MachineGroup
        base_machine = cls.__new__(cls)
        rows = ["max_price_hour", "max_vcpus", "max_instances"]
        base_machine.quota_usage = {k: before_quotas[k].in_use for k in rows}
        logging.info(base_machine.quota_usage_table_str("freed by resources"))

    return 0


def register(parser):
    """Register the terminate command for the resources."""

    subparser = parser.add_parser("terminate",
                                  help="Terminate resources.",
                                  formatter_class=argparse.RawTextHelpFormatter)

    subparser.description = textwrap.dedent("""\
        The `inductiva resources terminate` command provides a utility for
        terminating active computational resources. It allows you to specify
        the names of the resources to terminate, or terminate all active
        resources. Multiple resources can be terminated at once by providing
        their names. 

        All tasks running on the resources you are terminating will be
        immediately killed, regardless of their current stage.

        Each step requires user confirmation before proceeding.
    """)

    subparser.add_argument("name",
                           type=str,
                           help="Name(s) of the resource(s) to terminate.",
                           nargs="*")
    subparser.add_argument("-y",
                           "--yes",
                           action="store_true",
                           dest="confirm",
                           default=False,
                           help="Sets any confirmation values to \"yes\" "
                           "automatically. Users will not be asked for "
                           "confirmation to terminate resource(s).")
    subparser.add_argument("--all",
                           action="store_true",
                           help="Terminate all machines.")

    subparser.epilog = textwrap.dedent("""\
        examples:
            $ inductiva resources terminate --all
            You are about to terminate ALL resources.
            Are you sure you want to proceed (y/[N])? y
            Terminating MPICluster(name="api-p3kun5wyta1hacstu4xk38ujr"). This may take a few minutes.
            MPI Cluster api-p3kun5wyta1hacstu4xk38ujr with c2-standard-8 x2 machines successfully terminated in 0:01:10.
            Terminating MachineGroup(name="api-rdqprn82417bsd7id1qnac4c6"). This may take a few minutes.
            Machine Group api-rdqprn82417bsd7id1qnac4c6 with c2-standard-4 machines successfully terminated in 0:01:18.
    """)

    subparser.set_defaults(func=terminate_machine_group)



"""SFINCS module of the API."""
from typing import Optional, Union

from inductiva import simulators, tasks, types


class SFINCS(simulators.Simulator):
    """Class to invoke a generic SFINCS simulation on the API."""

    def __init__(self, /, version: Optional[str] = None, use_dev: bool = False):
        """Initialize the SFINCS simulator.

        Args:
            version (str): The version of the simulator to use. If None, the
                latest available version in the platform is used.
            use_dev (bool): Request use of the development version of
                the simulator. By default (False), the production version
                is used.
        """
        super().__init__(version=version, use_dev=use_dev)
        self.simulator = "arbitrary_commands"
        self.simulator_name_alias = "sfincs"

    def run(self,
            input_dir: Optional[str],
            *,
            on: types.ComputationalResources,
            storage_dir: Optional[str] = "",
            resubmit_on_preemption: bool = False,
            remote_assets: Optional[Union[str, list[str]]] = None,
            project: Optional[str] = None,
            time_to_live: Optional[str] = None,
            on_finish_cleanup: Optional[Union[str, list[str]]] = None,
            **kwargs) -> tasks.Task:
        """Run the simulation.

        Args:
            input_dir: Path to the directory of the simulation input files.
            on: The computational resource to launch the simulation on.
            sim_config_filename: Name of the simulation configuration file.
            storage_dir: Directory for storing simulation results.
            resubmit_on_preemption (bool): Resubmit task for execution when
                previous execution attempts were preempted. Only applicable when
                using a preemptible resource, i.e., resource instantiated with
                `spot=True`.
            remote_assets: Additional remote files that will be copied to
                the simulation directory.
            project: Name of the project to which the task will be
                assigned. If None, the task will be assigned to
                the default project. If the project does not exist, it will be
                created.
            time_to_live: Maximum allowed runtime for the task, specified as a
                string duration. Supports common time duration formats such as
                "10m", "2 hours", "1h30m", or "90s". The task will be
                automatically terminated if it exceeds this duration after
                starting.
            on_finish_cleanup :
                Optional cleanup script or list of shell commands to remove
                temporary or unwanted files generated during the simulation.
                This helps reduce storage usage by discarding unnecessary
                output.
                - If a string is provided, it is treated as the path to a shell
                script that must be included with the simulation files.
                - If a list of strings is provided, each item is treated as an
                individual shell command and will be executed sequentially.
                All cleanup actions are executed in the simulation's working
                directory, after the simulation finishes.
                Examples:
                    on_finish_cleanup = "my_cleanup.sh"

                    on_finish_cleanup = [
                        "rm -rf temp_dir",
                        "rm -f logs/debug.log"
                    ]
            other arguments: See the documentation of the base class.
        """

        self._input_files_exist(input_dir=input_dir,
                                remote_assets=remote_assets,
                                sim_config_filename="sfincs.inp")

        commands = ["sfincs"]

        return super().run(input_dir,
                           on=on,
                           commands=commands,
                           storage_dir=storage_dir,
                           resubmit_on_preemption=resubmit_on_preemption,
                           remote_assets=remote_assets,
                           project=project,
                           time_to_live=time_to_live,
                           on_finish_cleanup=on_finish_cleanup,
                           **kwargs)



# coding: utf-8

"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from inductiva.client.models.invitation_status import InvitationStatus
from inductiva.client.models.team_role import TeamRole
from typing import Optional, Set
from typing_extensions import Self


class TeamInvitationAdminResponse(BaseModel):
    """
    Schema for team invitation response (admin endpoints - full data).
    """ # noqa: E501
    email: StrictStr = Field(
        description="Email address of the person to invite")
    role: Optional[TeamRole] = None
    message: Optional[StrictStr] = None
    id: StrictStr
    organization_id: StrictStr
    status: InvitationStatus
    invitation_token: StrictStr
    invited_at: datetime
    expires_at: datetime
    accepted_at: Optional[datetime] = None
    rejected_at: Optional[datetime] = None
    invited_by_email: StrictStr
    invited_by_name: Optional[StrictStr] = None
    organization_name: StrictStr
    __properties: ClassVar[List[str]] = [
        "email", "role", "message", "id", "organization_id", "status",
        "invitation_token", "invited_at", "expires_at", "accepted_at",
        "rejected_at", "invited_by_email", "invited_by_name",
        "organization_name"
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TeamInvitationAdminResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if message (nullable) is None
        # and model_fields_set contains the field
        if self.message is None and "message" in self.model_fields_set:
            _dict['message'] = None

        # set to None if accepted_at (nullable) is None
        # and model_fields_set contains the field
        if self.accepted_at is None and "accepted_at" in self.model_fields_set:
            _dict['accepted_at'] = None

        # set to None if rejected_at (nullable) is None
        # and model_fields_set contains the field
        if self.rejected_at is None and "rejected_at" in self.model_fields_set:
            _dict['rejected_at'] = None

        # set to None if invited_by_name (nullable) is None
        # and model_fields_set contains the field
        if self.invited_by_name is None and "invited_by_name" in self.model_fields_set:
            _dict['invited_by_name'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TeamInvitationAdminResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "email": obj.get("email"),
            "role": obj.get("role"),
            "message": obj.get("message"),
            "id": obj.get("id"),
            "organization_id": obj.get("organization_id"),
            "status": obj.get("status"),
            "invitation_token": obj.get("invitation_token"),
            "invited_at": obj.get("invited_at"),
            "expires_at": obj.get("expires_at"),
            "accepted_at": obj.get("accepted_at"),
            "rejected_at": obj.get("rejected_at"),
            "invited_by_email": obj.get("invited_by_email"),
            "invited_by_name": obj.get("invited_by_name"),
            "organization_name": obj.get("organization_name")
        })
        return _obj



# coding: utf-8

"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from inductiva.client.models.orchestration_fee import OrchestrationFee
from inductiva.client.models.task_position_in_queue import TaskPositionInQueue
from inductiva.client.models.task_status_code import TaskStatusCode
from typing import Optional, Set
from typing_extensions import Self


class TaskSubmittedInfo(BaseModel):
    """
    TaskSubmittedInfo
    """

  # noqa: E501
    id: StrictStr
    status: TaskStatusCode
    position_in_queue: Optional[TaskPositionInQueue] = None
    is_terminated: StrictBool
    storage_region: Optional[StrictStr] = None
    time_to_live_seconds: Optional[Union[StrictFloat, StrictInt]] = None
    orchestration_fee: Optional[OrchestrationFee] = None
    __properties: ClassVar[List[str]] = [
        "id", "status", "position_in_queue", "is_terminated", "storage_region",
        "time_to_live_seconds", "orchestration_fee"
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TaskSubmittedInfo from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of position_in_queue
        if self.position_in_queue:
            _dict['position_in_queue'] = self.position_in_queue.to_dict()
        # override the default output from pydantic by calling `to_dict()` of orchestration_fee
        if self.orchestration_fee:
            _dict['orchestration_fee'] = self.orchestration_fee.to_dict()
        # set to None if position_in_queue (nullable) is None
        # and model_fields_set contains the field
        if self.position_in_queue is None and "position_in_queue" in self.model_fields_set:
            _dict['position_in_queue'] = None

        # set to None if storage_region (nullable) is None
        # and model_fields_set contains the field
        if self.storage_region is None and "storage_region" in self.model_fields_set:
            _dict['storage_region'] = None

        # set to None if time_to_live_seconds (nullable) is None
        # and model_fields_set contains the field
        if self.time_to_live_seconds is None and "time_to_live_seconds" in self.model_fields_set:
            _dict['time_to_live_seconds'] = None

        # set to None if orchestration_fee (nullable) is None
        # and model_fields_set contains the field
        if self.orchestration_fee is None and "orchestration_fee" in self.model_fields_set:
            _dict['orchestration_fee'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TaskSubmittedInfo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id":
                obj.get("id"),
            "status":
                obj.get("status"),
            "position_in_queue":
                TaskPositionInQueue.from_dict(obj["position_in_queue"])
                if obj.get("position_in_queue") is not None else None,
            "is_terminated":
                obj.get("is_terminated"),
            "storage_region":
                obj.get("storage_region"),
            "time_to_live_seconds":
                obj.get("time_to_live_seconds"),
            "orchestration_fee":
                OrchestrationFee.from_dict(obj["orchestration_fee"])
                if obj.get("orchestration_fee") is not None else None
        })
        return _obj



# coding: utf-8

"""
    Machine Catalogue

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import StrictStr
from typing import Optional
from inductiva.machines_catalogue_client.models.available_machine_series import AvailableMachineSeries
from inductiva.machines_catalogue_client.models.available_zones import AvailableZones
from inductiva.machines_catalogue_client.models.cloud_location import CloudLocation
from inductiva.machines_catalogue_client.models.providers import Providers
from inductiva.machines_catalogue_client.models.validation_result import ValidationResult

from inductiva.machines_catalogue_client.api_client import ApiClient, RequestSerialized
from inductiva.machines_catalogue_client.api_response import ApiResponse
from inductiva.machines_catalogue_client.rest import RESTResponseType


class ComputeApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    def get_available_machine_series_compute_available_machine_series_get(
        self,
        zone: StrictStr,
        provider_id: Providers,
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> AvailableMachineSeries:
        """Get Available Machine Series


        :param zone: (required)
        :type zone: str
        :param provider_id: (required)
        :type provider_id: Providers
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_available_machine_series_compute_available_machine_series_get_serialize(
            zone=zone,
            provider_id=provider_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AvailableMachineSeries",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def get_available_machine_series_compute_available_machine_series_get_with_http_info(
        self,
        zone: StrictStr,
        provider_id: Providers,
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[AvailableMachineSeries]:
        """Get Available Machine Series


        :param zone: (required)
        :type zone: str
        :param provider_id: (required)
        :type provider_id: Providers
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_available_machine_series_compute_available_machine_series_get_serialize(
            zone=zone,
            provider_id=provider_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AvailableMachineSeries",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def get_available_machine_series_compute_available_machine_series_get_without_preload_content(
        self,
        zone: StrictStr,
        provider_id: Providers,
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Available Machine Series


        :param zone: (required)
        :type zone: str
        :param provider_id: (required)
        :type provider_id: Providers
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_available_machine_series_compute_available_machine_series_get_serialize(
            zone=zone,
            provider_id=provider_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AvailableMachineSeries",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        return response_data.response

    def _get_available_machine_series_compute_available_machine_series_get_serialize(
        self,
        zone,
        provider_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes, List[str], List[bytes],
                                List[Tuple[str, bytes]]]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if zone is not None:

            _query_params.append(('zone', zone))

        if provider_id is not None:

            _query_params.append(('provider_id', provider_id.value))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                ['application/json'])

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/compute/available-machine-series',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth)

    @validate_call
    def get_available_zones_compute_available_zones_get(
        self,
        provider_id: Providers,
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> AvailableZones:
        """Get Available Zones


        :param provider_id: (required)
        :type provider_id: Providers
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_available_zones_compute_available_zones_get_serialize(
            provider_id=provider_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AvailableZones",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def get_available_zones_compute_available_zones_get_with_http_info(
        self,
        provider_id: Providers,
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[AvailableZones]:
        """Get Available Zones


        :param provider_id: (required)
        :type provider_id: Providers
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_available_zones_compute_available_zones_get_serialize(
            provider_id=provider_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AvailableZones",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def get_available_zones_compute_available_zones_get_without_preload_content(
        self,
        provider_id: Providers,
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Available Zones


        :param provider_id: (required)
        :type provider_id: Providers
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_available_zones_compute_available_zones_get_serialize(
            provider_id=provider_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AvailableZones",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        return response_data.response

    def _get_available_zones_compute_available_zones_get_serialize(
        self,
        provider_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes, List[str], List[bytes],
                                List[Tuple[str, bytes]]]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if provider_id is not None:

            _query_params.append(('provider_id', provider_id.value))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                ['application/json'])

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/compute/available-zones',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth)

    @validate_call
    def get_nearest_cloud_location_compute_nearest_cloud_location_get(
        self,
        country: StrictStr,
        provider_id: Providers,
        city: Optional[StrictStr] = None,
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CloudLocation:
        """Get Nearest Cloud Location


        :param country: (required)
        :type country: str
        :param provider_id: (required)
        :type provider_id: Providers
        :param city:
        :type city: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_nearest_cloud_location_compute_nearest_cloud_location_get_serialize(
            country=country,
            provider_id=provider_id,
            city=city,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudLocation",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def get_nearest_cloud_location_compute_nearest_cloud_location_get_with_http_info(
        self,
        country: StrictStr,
        provider_id: Providers,
        city: Optional[StrictStr] = None,
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CloudLocation]:
        """Get Nearest Cloud Location


        :param country: (required)
        :type country: str
        :param provider_id: (required)
        :type provider_id: Providers
        :param city:
        :type city: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_nearest_cloud_location_compute_nearest_cloud_location_get_serialize(
            country=country,
            provider_id=provider_id,
            city=city,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudLocation",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def get_nearest_cloud_location_compute_nearest_cloud_location_get_without_preload_content(
        self,
        country: StrictStr,
        provider_id: Providers,
        city: Optional[StrictStr] = None,
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Nearest Cloud Location


        :param country: (required)
        :type country: str
        :param provider_id: (required)
        :type provider_id: Providers
        :param city:
        :type city: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_nearest_cloud_location_compute_nearest_cloud_location_get_serialize(
            country=country,
            provider_id=provider_id,
            city=city,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudLocation",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        return response_data.response

    def _get_nearest_cloud_location_compute_nearest_cloud_location_get_serialize(
        self,
        country,
        provider_id,
        city,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes, List[str], List[bytes],
                                List[Tuple[str, bytes]]]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if country is not None:

            _query_params.append(('country', country))

        if city is not None:

            _query_params.append(('city', city))

        if provider_id is not None:

            _query_params.append(('provider_id', provider_id.value))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                ['application/json'])

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/compute/nearest-cloud-location',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth)

    @validate_call
    def validate_machine_type_zone_compute_validate_machine_type_zone_get(
        self,
        zone: StrictStr,
        machine_type: StrictStr,
        provider_id: Providers,
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ValidationResult:
        """Validate Machine Type Zone


        :param zone: (required)
        :type zone: str
        :param machine_type: (required)
        :type machine_type: str
        :param provider_id: (required)
        :type provider_id: Providers
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._validate_machine_type_zone_compute_validate_machine_type_zone_get_serialize(
            zone=zone,
            machine_type=machine_type,
            provider_id=provider_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ValidationResult",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def validate_machine_type_zone_compute_validate_machine_type_zone_get_with_http_info(
        self,
        zone: StrictStr,
        machine_type: StrictStr,
        provider_id: Providers,
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ValidationResult]:
        """Validate Machine Type Zone


        :param zone: (required)
        :type zone: str
        :param machine_type: (required)
        :type machine_type: str
        :param provider_id: (required)
        :type provider_id: Providers
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._validate_machine_type_zone_compute_validate_machine_type_zone_get_serialize(
            zone=zone,
            machine_type=machine_type,
            provider_id=provider_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ValidationResult",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def validate_machine_type_zone_compute_validate_machine_type_zone_get_without_preload_content(
        self,
        zone: StrictStr,
        machine_type: StrictStr,
        provider_id: Providers,
        _request_timeout: Union[None, Annotated[StrictFloat,
                                                Field(gt=0)],
                                Tuple[Annotated[StrictFloat,
                                                Field(gt=0)],
                                      Annotated[StrictFloat,
                                                Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Validate Machine Type Zone


        :param zone: (required)
        :type zone: str
        :param machine_type: (required)
        :type machine_type: str
        :param provider_id: (required)
        :type provider_id: Providers
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._validate_machine_type_zone_compute_validate_machine_type_zone_get_serialize(
            zone=zone,
            machine_type=machine_type,
            provider_id=provider_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index)

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ValidationResult",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout)
        return response_data.response

    def _validate_machine_type_zone_compute_validate_machine_type_zone_get_serialize(
        self,
        zone,
        machine_type,
        provider_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes, List[str], List[bytes],
                                List[Tuple[str, bytes]]]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if zone is not None:

            _query_params.append(('zone', zone))

        if machine_type is not None:

            _query_params.append(('machine_type', machine_type))

        if provider_id is not None:

            _query_params.append(('provider_id', provider_id.value))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                ['application/json'])

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/compute/validate-machine-type-zone',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth)



"""
List all container files inside a specified (or default) storage folder.
"""

import argparse
from inductiva import storage, client
import textwrap


def list_containers(args):
    default_folder = "my-containers"
    output_path = args.folder

    if not output_path:
        output_path = default_folder

    try:
        storage.listdir(output_path, max_results=args.max_results)
    except client.exceptions.ApiException as e:
        if e.status == 404:
            error_msg = f"Folder '{output_path}' not found on remote storage."
            if output_path == default_folder:
                error_msg += ("\nUse the `inductiva containers upload` command "
                              "to upload a container to the remote storage.")
            print(error_msg)
            return False
    except Exception:  # pylint: disable=broad-exception-caught
        print("Unkown error while listing containers.")
        return False


def register(parser):
    """Register the upload-container command."""
    subparser = parser.add_parser(
        "list",
        aliases=["ls"],
        help=("List all container files in remote storage, "
              "including their size and estimated cost."),
        formatter_class=argparse.RawTextHelpFormatter)

    subparser.description = textwrap.dedent("""\
        The `inductiva containers list` command lists all container files in
        remote storage under the default containers folder (`my-containers/`),
        including their size and estimated cost.
        
        You can also specify other container folders to list their contents.
    """)

    subparser.add_argument(
        "folder",
        nargs="?",
        type=str,
        help="Path to a containers folder in remote storage.",
    )

    subparser.add_argument(
        "--max-results",
        "-m",
        default=10,
        type=int,
        help="Maximum number of container files to list.",
    )

    subparser.epilog = textwrap.dedent("""\
        examples:
            $ inductiva containers list
            NAME             SIZE        CREATION TIME
            container1.sif   200.00 MB   26/03, 16:41:14
            container2.sif   100.00 MB   26/03, 16:41:14
            container3.sif   300.00 MB   26/03, 16:41:14

            Total storage size used:
                    Volume: 0.59 GB
                    Cost: 0.02 US$/month
    """)

    subparser.set_defaults(func=list_containers)



"""Register CLI commands for storage."""
import argparse
import os
import textwrap

from inductiva import constants
from inductiva._cli import loader, utils


def register(root_parser):

    parser = root_parser.add_parser(
        "storage",
        help="Remote storage management utilities.",
        formatter_class=argparse.RawTextHelpFormatter)

    parser.description = textwrap.dedent("""\
        Remote storage management utilities. 
                                        
        Use the `inductiva storage` command to manage your data in Inductiva's
        remote storage. It supports a range of operations including listing
        files, downloading and deleting items, and calculating total storage
        usage and cost.
    """)

    utils.show_help_msg(parser)

    subparsers = parser.add_subparsers(title="available subcomands")
    loader.load_commands(subparsers,
                         os.path.dirname(__file__),
                         package=__name__,
                         ignores_prefix=constants.LOADER_IGNORE_PREFIX,
                         hides_prefix=constants.LOADER_HIDE_PREFIX)



"""SplisHSPlasH simulator module of the API."""
from typing import Optional, Tuple, Union

from inductiva import simulators, tasks, types


class SplishSplash(simulators.Simulator):
    """Class to invoke a generic SPlisHSPlasH simulation on the API."""

    def __init__(self, /, version: Optional[str] = None, use_dev: bool = False):
        """Initialize the SPlisHSplasH simulator.

        Args:
            version (str): The version of the simulator to use. If None, the
                latest available version in the platform is used.
            use_dev (bool): Request use of the development version of
                the simulator. By default (False), the production version
                is used.
        """
        super().__init__(version=version, use_dev=use_dev)
        self.simulator = "arbitrary_commands"
        self.simulator_name_alias = "splishsplash"

    def run(
        self,
        input_dir: Optional[str],
        sim_config_filename: str,
        *,
        on: types.ComputationalResources,
        storage_dir: Optional[str] = "",
        resubmit_on_preemption: bool = False,
        remote_assets: Optional[Union[str, list[str]]] = None,
        project: Optional[str] = None,
        time_to_live: Optional[str] = None,
        vtk_to_obj: Optional[bool] = False,
        vtk_to_obj_vtk_dir: Optional[str] = None,
        vtk_to_obj_vtk_prefix: Optional[str] = "PartFluid_",
        vtk_to_obj_out_dir: Optional[str] = None,
        vtk_to_obj_particle_radius: Optional[float] = None,
        vtk_to_obj_smoothing_length: Optional[float] = 2.0,
        vtk_to_obj_cube_size: Optional[float] = 1.0,
        vtk_to_obj_surface_threshold: Optional[float] = 0.6,
        gen_gif: Optional[bool] = False,
        gen_gif_cam_pos: Tuple[float, float, float] = (4.0, 1.0, 4.0),
        gen_gif_cam_fp: Tuple[float, float, float] = (0.0, 0.0, 0.0),
        on_finish_cleanup: Optional[Union[str, list[str]]] = None,
        **kwargs,
    ) -> tasks.Task:
        """Run the SPlisHSPlasH simulation.

        Args:
            input_dir: Path to the directory of the simulation input files.
            sim_config_filename: Name of the simulation configuration file.
            on: The computational resource to launch the simulation on. If None
                the simulation is submitted to a machine in the default pool.
            storage_dir: Directory for storing simulation results.
            resubmit_on_preemption (bool): Resubmit task for execution when
                previous execution attempts were preempted. Only applicable when
                using a preemptible resource, i.e., resource instantiated with
                `spot=True`.
            remote_assets: Additional remote files that will be copied to
                the simulation directory.
            project: Name of the project to which the task will be
                assigned. If None, the task will be assigned to
                the default project. If the project does not exist, it will be
                created.
            time_to_live: Maximum allowed runtime for the task, specified as a
                string duration. Supports common time duration formats such as
                "10m", "2 hours", "1h30m", or "90s". The task will be
                automatically terminated if it exceeds this duration after
                starting.
            vtk_to_obj: Whether to convert the output VTK files to OBJ meshes
                using marching cubes.
            vtk_to_obj_vtk_dir: Directory containing VTK files to be converted.
            vtk_to_obj_out_dir: Directory where the generated OBJ files will be
                stored.
            vtk_to_obj_vtk_prefix: Prefix of the VTK files that will be
                converted.
                Default: PartFluid_ 
            vtk_to_obj_particle_radius: The particle radius of the input data.
            vtk_to_obj_smoothing_length: The smoothing length radius used for
                the SPH kernel, the kernel compact support radius will be twice
                the smoothing length (in multiplies of the particle radius).
                Default: 2.0
            vtk_to_obj_cube_size: The cube edge length used for marching cubes
                in multiplies of the particle radius, corresponds to the cell
                size of the implicit background grid.
                Default: 1.0
            vtk_to_obj_surface_threshold: The iso-surface threshold for the
                density, i.e. the normalized value of the reconstructed density
                level that indicates the fluid surface (in multiplies of the
                rest density).
                Default: 0.6
            gen_gif: Whether to generate an animated GIF from the
                simulation output.
            gen_gif_cam_pos: The position of the camera when generating the GIF.  
                Default: (4.0, 1.0, 4.0).
            gen_gif_cam_fp: The point in space the camera looks at (focus
                point).  
                Default: (0.0, 0.0, 0.0).
            on_finish_cleanup :
                Optional cleanup script or list of shell commands to remove
                temporary or unwanted files generated during the simulation.
                This helps reduce storage usage by discarding unnecessary
                output.
                - If a string is provided, it is treated as the path to a shell
                script that must be included with the simulation files.
                - If a list of strings is provided, each item is treated as an
                individual shell command and will be executed sequentially.
                All cleanup actions are executed in the simulation's working
                directory, after the simulation finishes.
                Examples:
                    on_finish_cleanup = "my_cleanup.sh"

                    on_finish_cleanup = [
                        "rm -rf temp_dir",
                        "rm -f logs/debug.log"
                    ]
        Returns:
            Task object representing the simulation task.
        """

        self._input_files_exist(input_dir=input_dir,
                                remote_assets=remote_assets,
                                sim_config_filename=sim_config_filename)

        if vtk_to_obj and (vtk_to_obj_vtk_dir is None or
                           vtk_to_obj_particle_radius is None):
            raise ValueError("When using `vtk_to_obj=True`, "
                             "`vtk_to_obj_vtk_dir` and "
                             "`vtk_to_obj_particle_radius` need to be defined.")

        commands = [
            "cp /SPlisHSPlasH_CPU/bin/SPHSimulator .",
            f"./SPHSimulator {sim_config_filename} --no-gui --output-dir .",
            "rm SPHSimulator"
        ]

        if vtk_to_obj:

            # If out_dir is not provided, will save in the same directory as
            # the vtk files
            if vtk_to_obj_out_dir is None:
                vtk_to_obj_out_dir = vtk_to_obj_vtk_dir

            commands.append(
                "splashsurf reconstruct "
                f"{vtk_to_obj_vtk_dir}/{vtk_to_obj_vtk_prefix}"
                "{}.vtk "
                f"-r={vtk_to_obj_particle_radius} "
                f"-l={vtk_to_obj_smoothing_length} "
                f"-c={vtk_to_obj_cube_size} "
                f"-t={vtk_to_obj_surface_threshold} "
                "--subdomain-grid=on --mesh-cleanup=on "
                "--mesh-smoothing-weights=on --mesh-smoothing-iters=25 "
                "--normals=on --normals-smoothing-iters=10 "
                f"-o {vtk_to_obj_out_dir}/{vtk_to_obj_vtk_prefix}_surface"
                "{}.obj")
        if gen_gif:
            commands.append("python3 /home/scripts/gen_gif.py ./vtk res.gif "
                            f"--cam_pos {gen_gif_cam_pos[0]} "
                            f"{gen_gif_cam_pos[1]} "
                            f"{gen_gif_cam_pos[2]} "
                            f"--cam_fp {gen_gif_cam_fp[0]} "
                            f"{gen_gif_cam_fp[1]} "
                            f"{gen_gif_cam_fp[2]}")

        return super().run(
            input_dir,
            commands=commands,
            storage_dir=storage_dir,
            on=on,
            resubmit_on_preemption=resubmit_on_preemption,
            remote_assets=remote_assets,
            project=project,
            time_to_live=time_to_live,
            on_finish_cleanup=on_finish_cleanup,
            **kwargs,
        )



"""Launches a Task-Runner via CLI."""
from typing import TextIO
import argparse
import threading
import sys
import os
import platform
from inductiva import _cli, constants, _api_key, api_url
from inductiva.resources import machine_groups

_docker_imported = True
try:
    import docker
    from docker.errors import DockerException
except ImportError:
    _docker_imported = False


def generator_thread(container, fout: TextIO):
    """Runs a generator in a separate thread."""
    for line in container.logs(stream=True):
        print(line.decode("utf-8"), end="", file=fout)


def join_container_streams(*containers, fout: TextIO = sys.stdout):
    """Join multiple containers into one."""
    container_threads = []

    for container in containers:
        thread = threading.Thread(target=generator_thread,
                                  args=(container, fout))
        thread.start()
        container_threads.append(thread)

    for thread in container_threads:
        thread.join()


def launch_task_runner_gcp(args):
    """Launches a Task-Runner on GCP.
    
    Note: BYOC (Bring Your Own Cloud) machine groups can only have a single VM.
    """
    machine_groups.MachineGroup(
        machine_type=args.machine_type,
        zone=args.zone,
        provider="GCP",
        spot=args.spot,
        mg_name=args.machine_group_name,
        data_disk_gb=args.disk_size,
        byoc=True,
        max_idle_time=args.max_idle_time,
    ).start()


def launch_task_runner(args, fout: TextIO = sys.stdout):
    """Launches a Task-Runner."""
    if args.provider == "gcp":
        launch_task_runner_gcp(args)
    else:  # local
        launch_task_runner_local(args, fout)


def launch_task_runner_local(args, fout: TextIO = sys.stdout):
    """Launches a Task-Runner locally using Docker."""
    if not _docker_imported:
        print(
            "Docker Python API not installed, please run "
            "'pip install \"inductiva[task-runner]\"' to install it",
            file=fout)
        return

    try:
        client = docker.from_env()
    except DockerException as e:
        print(f"Failed to connect to Docker: {e}", file=fout)
        print(
            "Please make sure Docker is running "
            "and you have the necessary permissions.",
            file=fout)
        return

    task_runner = client.containers.list(filters={"name": "task-runner"})
    file_tracker = client.containers.list(filters={"name": "file-tracker"})

    if task_runner or file_tracker:
        print(
            "Task-Runner already running. "
            "Please stop it before launching a new one.",
            file=fout)
        return

    client.images.pull(constants.FILE_TRACKER_IMAGE, platform="linux/amd64")
    file_tracker_container = client.containers.run(
        image=constants.FILE_TRACKER_IMAGE,
        name="file-tracker",
        environment={
            "API_URL": api_url,
            "USER_API_KEY": _api_key.get(),
        },
        volumes={
            "workdir": {
                "bind": "/workdir",
                "mode": "rw"
            },
        },
        network="host",
        platform="linux/amd64",
        detach=True,
        auto_remove=True,
    )

    client.images.pull(constants.TASK_RUNNER_IMAGE, platform="linux/amd64")

    apptainer_path = "apptainer"
    os.makedirs(apptainer_path, exist_ok=True)
    os.chmod(apptainer_path, 0o777)
    apptainer_full_path = os.path.abspath(apptainer_path)

    hostname = args.hostname or platform.uname().node

    task_runner_container = client.containers.run(
        image=constants.TASK_RUNNER_IMAGE,
        name="task-runner",
        environment={
            "USER_API_KEY": _api_key.get(),
            "API_URL": api_url,
            "MACHINE_GROUP_NAME": args.machine_group_name,
            "HOST_NAME": hostname,
        },
        mounts=[
            docker.types.Mount(target="/executer-images",
                               source=apptainer_full_path,
                               type="bind")
        ],
        volumes={
            "workdir": {
                "bind": "/workdir",
                "mode": "rw"
            },
        },
        network="host",
        privileged=True,
        platform="linux/amd64",
        detach=True,
        auto_remove=True,
    )

    print(
        "File-Tracker launched "
        f"with container ID: {file_tracker_container.short_id}",
        file=fout)
    print(
        "Task-Runner launched "
        f"with container ID: {task_runner_container.short_id}",
        file=fout)

    if not args.detach:
        try:
            join_container_streams(task_runner_container,
                                   file_tracker_container)
        except KeyboardInterrupt:
            print("Interrupted. Stopping containers...", file=fout)
            file_tracker_container.stop()
            task_runner_container.stop()


def register(parser):
    """Register the launch task-runner command."""
    subparser = parser.add_parser("launch",
                                  help="Launches a Task-Runner.",
                                  formatter_class=argparse.RawTextHelpFormatter)

    subparser.description = (
        "The `inductiva task-runner launch` command provides "
        "a way to launch a Task-Runner on different providers.\n\n"
        "Providers:\n"
        "  local  - Launch locally using Docker (default)\n"
        "  gcp    - Launch on Google Cloud Platform")

    _cli.utils.add_watch_argument(subparser)

    subparser.add_argument(
        "--provider",
        "-p",
        choices=["local", "gcp"],
        default="local",
        help="Provider to use for launching the task-runner (default: local).")

    subparser.add_argument(
        "machine_group_name",
        type=str,
        help="Name of the machine group to launch the Task-Runner.")

    subparser.add_argument("--hostname",
                           "-ho",
                           type=str,
                           help="Hostname of the Task-Runner.")

    subparser.add_argument(
        "--detach",
        "-d",
        action="store_true",
        help="Run the task-runner in the background (local only).")

    # GCP-specific arguments
    gcp_group = subparser.add_argument_group("GCP-specific options")
    gcp_group.add_argument(
        "--zone",
        "-z",
        type=str,
        default="europe-west1-b",
        help="GCP zone where the VM will be created (default: europe-west1-b).")

    gcp_group.add_argument("--machine-type",
                           "-t",
                           type=str,
                           default="c2d-standard-8",
                           help="GCP machine type (default: c2d-standard-8).")

    gcp_group.add_argument("--spot",
                           action="store_true",
                           help="Use spot instance.")

    gcp_group.add_argument("--max-idle-time",
                           type=int,
                           help="Idle time in minutes before "
                           "auto-termination (default: 3).")

    gcp_group.add_argument("--disk-size",
                           "-ds",
                           type=int,
                           default=10,
                           help="Disk size in GB (default: 10).")

    subparser.set_defaults(func=launch_task_runner)
